// Code generated by smithy-go-codegen DO NOT EDIT.

package codeartifact

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/codeartifact/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Returns the most closely associated package group to the specified package.
// This API does not require that the package exist in any repository in the
// domain. As such, GetAssociatedPackageGroup can be used to see which package
// group's origin configuration applies to a package before that package is in a
// repository. This can be helpful to check if public packages are blocked without
// ingesting them. For information package group association and matching, see
// Package group definition syntax and matching behavior (https://docs.aws.amazon.com/codeartifact/latest/ug/package-group-definition-syntax-matching-behavior.html)
// in the CodeArtifact User Guide.
func (c *Client) GetAssociatedPackageGroup(ctx context.Context, params *GetAssociatedPackageGroupInput, optFns ...func(*Options)) (*GetAssociatedPackageGroupOutput, error) {
	if params == nil {
		params = &GetAssociatedPackageGroupInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "GetAssociatedPackageGroup", params, optFns, c.addOperationGetAssociatedPackageGroupMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*GetAssociatedPackageGroupOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type GetAssociatedPackageGroupInput struct {

	// The name of the domain that contains the package from which to get the
	// associated package group.
	//
	// This member is required.
	Domain *string

	// The format of the package from which to get the associated package group.
	//
	// This member is required.
	Format types.PackageFormat

	// The package from which to get the associated package group.
	//
	// This member is required.
	Package *string

	// The 12-digit account number of the Amazon Web Services account that owns the
	// domain. It does not include dashes or spaces.
	DomainOwner *string

	// The namespace of the package from which to get the associated package group.
	// The package component that specifies its namespace depends on its type. For
	// example: The namespace is required when getting associated package groups from
	// packages of the following formats:
	//   - Maven
	//   - Swift
	//   - generic
	//
	//   - The namespace of a Maven package version is its groupId .
	//   - The namespace of an npm or Swift package version is its scope .
	//   - The namespace of a generic package is its namespace .
	//   - Python and NuGet package versions do not contain a corresponding component,
	//   package versions of those formats do not have a namespace.
	Namespace *string

	noSmithyDocumentSerde
}

type GetAssociatedPackageGroupOutput struct {

	// Describes the strength of the association between the package and package
	// group. A strong match is also known as an exact match, and a weak match is known
	// as a relative match.
	AssociationType types.PackageGroupAssociationType

	// The package group that is associated with the requested package.
	PackageGroup *types.PackageGroupDescription

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationGetAssociatedPackageGroupMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsRestjson1_serializeOpGetAssociatedPackageGroup{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpGetAssociatedPackageGroup{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "GetAssociatedPackageGroup"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addOpGetAssociatedPackageGroupValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opGetAssociatedPackageGroup(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	return nil
}

func newServiceMetadataMiddleware_opGetAssociatedPackageGroup(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "GetAssociatedPackageGroup",
	}
}
