// Code generated by smithy-go-codegen DO NOT EDIT.

package medialive

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/medialive/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Updates the parameters for the input device.
func (c *Client) UpdateInputDevice(ctx context.Context, params *UpdateInputDeviceInput, optFns ...func(*Options)) (*UpdateInputDeviceOutput, error) {
	if params == nil {
		params = &UpdateInputDeviceInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "UpdateInputDevice", params, optFns, c.addOperationUpdateInputDeviceMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*UpdateInputDeviceOutput)
	out.ResultMetadata = metadata
	return out, nil
}

// A request to update an input device.
type UpdateInputDeviceInput struct {

	// The unique ID of the input device. For example, hd-123456789abcdef.
	//
	// This member is required.
	InputDeviceId *string

	// The Availability Zone you want associated with this input device.
	AvailabilityZone *string

	// The settings that you want to apply to the HD input device.
	HdDeviceSettings *types.InputDeviceConfigurableSettings

	// The name that you assigned to this input device (not the unique ID).
	Name *string

	// The settings that you want to apply to the UHD input device.
	UhdDeviceSettings *types.InputDeviceConfigurableSettings

	noSmithyDocumentSerde
}

// Placeholder documentation for UpdateInputDeviceResponse
type UpdateInputDeviceOutput struct {

	// The unique ARN of the input device.
	Arn *string

	// The Availability Zone associated with this input device.
	AvailabilityZone *string

	// The state of the connection between the input device and AWS.
	ConnectionState types.InputDeviceConnectionState

	// The status of the action to synchronize the device configuration. If you change
	// the configuration of the input device (for example, the maximum bitrate),
	// MediaLive sends the new data to the device. The device might not update itself
	// immediately. SYNCED means the device has updated its configuration. SYNCING
	// means that it has not updated its configuration.
	DeviceSettingsSyncState types.DeviceSettingsSyncState

	// The status of software on the input device.
	DeviceUpdateStatus types.DeviceUpdateStatus

	// Settings that describe an input device that is type HD.
	HdDeviceSettings *types.InputDeviceHdSettings

	// The unique ID of the input device.
	Id *string

	// The network MAC address of the input device.
	MacAddress *string

	// An array of the ARNs for the MediaLive inputs attached to the device. Returned
	// only if the outputType is MEDIALIVE_INPUT.
	MedialiveInputArns []string

	// A name that you specify for the input device.
	Name *string

	// The network settings for the input device.
	NetworkSettings *types.InputDeviceNetworkSettings

	// The output attachment type of the input device. Specifies MEDIACONNECT_FLOW if
	// this device is the source for a MediaConnect flow. Specifies MEDIALIVE_INPUT if
	// this device is the source for a MediaLive input.
	OutputType types.InputDeviceOutputType

	// The unique serial number of the input device.
	SerialNumber *string

	// A collection of key-value pairs.
	Tags map[string]string

	// The type of the input device.
	Type types.InputDeviceType

	// Settings that describe an input device that is type UHD.
	UhdDeviceSettings *types.InputDeviceUhdSettings

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationUpdateInputDeviceMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsRestjson1_serializeOpUpdateInputDevice{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpUpdateInputDevice{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "UpdateInputDevice"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addOpUpdateInputDeviceValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opUpdateInputDevice(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	return nil
}

func newServiceMetadataMiddleware_opUpdateInputDevice(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "UpdateInputDevice",
	}
}
