// Code generated by smithy-go-codegen DO NOT EDIT.

package neptunedata

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/neptunedata/document"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Gets status information about a specified load job. Neptune keeps track of the
// most recent 1,024 bulk load jobs, and stores the last 10,000 error details per
// job. See Neptune Loader Get-Status API (https://docs.aws.amazon.com/neptune/latest/userguide/load-api-reference-status.htm)
// for more information. When invoking this operation in a Neptune cluster that has
// IAM authentication enabled, the IAM user or role making the request must have a
// policy attached that allows the neptune-db:GetLoaderJobStatus (https://docs.aws.amazon.com/neptune/latest/userguide/iam-dp-actions.html#getloaderjobstatus)
// IAM action in that cluster..
func (c *Client) GetLoaderJobStatus(ctx context.Context, params *GetLoaderJobStatusInput, optFns ...func(*Options)) (*GetLoaderJobStatusOutput, error) {
	if params == nil {
		params = &GetLoaderJobStatusInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "GetLoaderJobStatus", params, optFns, c.addOperationGetLoaderJobStatusMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*GetLoaderJobStatusOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type GetLoaderJobStatusInput struct {

	// The load ID of the load job to get the status of.
	//
	// This member is required.
	LoadId *string

	// Flag indicating whether or not to include details beyond the overall status (
	// TRUE or FALSE ; the default is FALSE ).
	Details *bool

	// Flag indicating whether or not to include a list of errors encountered ( TRUE
	// or FALSE ; the default is FALSE ). The list of errors is paged. The page and
	// errorsPerPage parameters allow you to page through all the errors.
	Errors *bool

	// The number of errors returned in each page (a positive integer; the default is
	// 10 ). Only valid when the errors parameter set to TRUE .
	ErrorsPerPage *int32

	// The error page number (a positive integer; the default is 1 ). Only valid when
	// the errors parameter is set to TRUE .
	Page *int32

	noSmithyDocumentSerde
}

type GetLoaderJobStatusOutput struct {

	// Status information about the load job, in a layout that could look like this:
	//
	// This member is required.
	Payload document.Interface

	// The HTTP response code for the request.
	//
	// This member is required.
	Status *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationGetLoaderJobStatusMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsRestjson1_serializeOpGetLoaderJobStatus{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpGetLoaderJobStatus{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "GetLoaderJobStatus"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addOpGetLoaderJobStatusValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opGetLoaderJobStatus(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	return nil
}

func newServiceMetadataMiddleware_opGetLoaderJobStatus(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "GetLoaderJobStatus",
	}
}
