// Code generated by smithy-go-codegen DO NOT EDIT.

package qbusiness

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/qbusiness/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Lists one or more Amazon Q conversations.
func (c *Client) ListConversations(ctx context.Context, params *ListConversationsInput, optFns ...func(*Options)) (*ListConversationsOutput, error) {
	if params == nil {
		params = &ListConversationsInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "ListConversations", params, optFns, c.addOperationListConversationsMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*ListConversationsOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type ListConversationsInput struct {

	// The identifier of the Amazon Q application.
	//
	// This member is required.
	ApplicationId *string

	// The identifier of the user involved in the Amazon Q web experience conversation.
	//
	// This member is required.
	UserId *string

	// The maximum number of Amazon Q conversations to return.
	MaxResults *int32

	// If the maxResults response was incomplete because there is more data to
	// retrieve, Amazon Q returns a pagination token in the response. You can use this
	// pagination token to retrieve the next set of Amazon Q conversations.
	NextToken *string

	noSmithyDocumentSerde
}

type ListConversationsOutput struct {

	// An array of summary information on the configuration of one or more Amazon Q
	// web experiences.
	Conversations []types.Conversation

	// If the response is truncated, Amazon Q returns this token, which you can use in
	// a later request to list the next set of messages.
	NextToken *string

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationListConversationsMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsRestjson1_serializeOpListConversations{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpListConversations{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "ListConversations"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addOpListConversationsValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opListConversations(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	return nil
}

// ListConversationsAPIClient is a client that implements the ListConversations
// operation.
type ListConversationsAPIClient interface {
	ListConversations(context.Context, *ListConversationsInput, ...func(*Options)) (*ListConversationsOutput, error)
}

var _ ListConversationsAPIClient = (*Client)(nil)

// ListConversationsPaginatorOptions is the paginator options for ListConversations
type ListConversationsPaginatorOptions struct {
	// The maximum number of Amazon Q conversations to return.
	Limit int32

	// Set to true if pagination should stop if the service returns a pagination token
	// that matches the most recent token provided to the service.
	StopOnDuplicateToken bool
}

// ListConversationsPaginator is a paginator for ListConversations
type ListConversationsPaginator struct {
	options   ListConversationsPaginatorOptions
	client    ListConversationsAPIClient
	params    *ListConversationsInput
	nextToken *string
	firstPage bool
}

// NewListConversationsPaginator returns a new ListConversationsPaginator
func NewListConversationsPaginator(client ListConversationsAPIClient, params *ListConversationsInput, optFns ...func(*ListConversationsPaginatorOptions)) *ListConversationsPaginator {
	if params == nil {
		params = &ListConversationsInput{}
	}

	options := ListConversationsPaginatorOptions{}
	if params.MaxResults != nil {
		options.Limit = *params.MaxResults
	}

	for _, fn := range optFns {
		fn(&options)
	}

	return &ListConversationsPaginator{
		options:   options,
		client:    client,
		params:    params,
		firstPage: true,
		nextToken: params.NextToken,
	}
}

// HasMorePages returns a boolean indicating whether more pages are available
func (p *ListConversationsPaginator) HasMorePages() bool {
	return p.firstPage || (p.nextToken != nil && len(*p.nextToken) != 0)
}

// NextPage retrieves the next ListConversations page.
func (p *ListConversationsPaginator) NextPage(ctx context.Context, optFns ...func(*Options)) (*ListConversationsOutput, error) {
	if !p.HasMorePages() {
		return nil, fmt.Errorf("no more pages available")
	}

	params := *p.params
	params.NextToken = p.nextToken

	var limit *int32
	if p.options.Limit > 0 {
		limit = &p.options.Limit
	}
	params.MaxResults = limit

	result, err := p.client.ListConversations(ctx, &params, optFns...)
	if err != nil {
		return nil, err
	}
	p.firstPage = false

	prevToken := p.nextToken
	p.nextToken = result.NextToken

	if p.options.StopOnDuplicateToken &&
		prevToken != nil &&
		p.nextToken != nil &&
		*prevToken == *p.nextToken {
		p.nextToken = nil
	}

	return result, nil
}

func newServiceMetadataMiddleware_opListConversations(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "ListConversations",
	}
}
