// Code generated by smithy-go-codegen DO NOT EDIT.

package quicksight

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/quicksight/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
)

// Lists all groups that are associated with a role.
func (c *Client) ListRoleMemberships(ctx context.Context, params *ListRoleMembershipsInput, optFns ...func(*Options)) (*ListRoleMembershipsOutput, error) {
	if params == nil {
		params = &ListRoleMembershipsInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "ListRoleMemberships", params, optFns, c.addOperationListRoleMembershipsMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*ListRoleMembershipsOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type ListRoleMembershipsInput struct {

	// The ID for the Amazon Web Services account that you want to create a group in.
	// The Amazon Web Services account ID that you provide must be the same Amazon Web
	// Services account that contains your Amazon QuickSight account.
	//
	// This member is required.
	AwsAccountId *string

	// The namespace that includes the role.
	//
	// This member is required.
	Namespace *string

	// The name of the role.
	//
	// This member is required.
	Role types.Role

	// The maximum number of results to return.
	MaxResults *int32

	// A pagination token that can be used in a subsequent request.
	NextToken *string

	noSmithyDocumentSerde
}

type ListRoleMembershipsOutput struct {

	// The list of groups associated with a role
	MembersList []string

	// A pagination token that can be used in a subsequent request.
	NextToken *string

	// The Amazon Web Services request ID for this operation.
	RequestId *string

	// The HTTP status of the request.
	Status int32

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationListRoleMembershipsMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsRestjson1_serializeOpListRoleMemberships{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsRestjson1_deserializeOpListRoleMemberships{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "ListRoleMemberships"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addOpListRoleMembershipsValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opListRoleMemberships(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	return nil
}

// ListRoleMembershipsAPIClient is a client that implements the
// ListRoleMemberships operation.
type ListRoleMembershipsAPIClient interface {
	ListRoleMemberships(context.Context, *ListRoleMembershipsInput, ...func(*Options)) (*ListRoleMembershipsOutput, error)
}

var _ ListRoleMembershipsAPIClient = (*Client)(nil)

// ListRoleMembershipsPaginatorOptions is the paginator options for
// ListRoleMemberships
type ListRoleMembershipsPaginatorOptions struct {
	// The maximum number of results to return.
	Limit int32

	// Set to true if pagination should stop if the service returns a pagination token
	// that matches the most recent token provided to the service.
	StopOnDuplicateToken bool
}

// ListRoleMembershipsPaginator is a paginator for ListRoleMemberships
type ListRoleMembershipsPaginator struct {
	options   ListRoleMembershipsPaginatorOptions
	client    ListRoleMembershipsAPIClient
	params    *ListRoleMembershipsInput
	nextToken *string
	firstPage bool
}

// NewListRoleMembershipsPaginator returns a new ListRoleMembershipsPaginator
func NewListRoleMembershipsPaginator(client ListRoleMembershipsAPIClient, params *ListRoleMembershipsInput, optFns ...func(*ListRoleMembershipsPaginatorOptions)) *ListRoleMembershipsPaginator {
	if params == nil {
		params = &ListRoleMembershipsInput{}
	}

	options := ListRoleMembershipsPaginatorOptions{}
	if params.MaxResults != nil {
		options.Limit = *params.MaxResults
	}

	for _, fn := range optFns {
		fn(&options)
	}

	return &ListRoleMembershipsPaginator{
		options:   options,
		client:    client,
		params:    params,
		firstPage: true,
		nextToken: params.NextToken,
	}
}

// HasMorePages returns a boolean indicating whether more pages are available
func (p *ListRoleMembershipsPaginator) HasMorePages() bool {
	return p.firstPage || (p.nextToken != nil && len(*p.nextToken) != 0)
}

// NextPage retrieves the next ListRoleMemberships page.
func (p *ListRoleMembershipsPaginator) NextPage(ctx context.Context, optFns ...func(*Options)) (*ListRoleMembershipsOutput, error) {
	if !p.HasMorePages() {
		return nil, fmt.Errorf("no more pages available")
	}

	params := *p.params
	params.NextToken = p.nextToken

	var limit *int32
	if p.options.Limit > 0 {
		limit = &p.options.Limit
	}
	params.MaxResults = limit

	result, err := p.client.ListRoleMemberships(ctx, &params, optFns...)
	if err != nil {
		return nil, err
	}
	p.firstPage = false

	prevToken := p.nextToken
	p.nextToken = result.NextToken

	if p.options.StopOnDuplicateToken &&
		prevToken != nil &&
		p.nextToken != nil &&
		*prevToken == *p.nextToken {
		p.nextToken = nil
	}

	return result, nil
}

func newServiceMetadataMiddleware_opListRoleMemberships(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "ListRoleMemberships",
	}
}
